==> jboss-4.0.1-src.tar.bz2: jboss-4.0.1-src/webservice/src/main/org/jboss/webservice/server/ServiceEndpointServlet.java <==
/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.webservice.server;

// $Id: ServiceEndpointServlet.java,v 1.8.2.2 2004/11/16 11:42:23 tdiesler Exp $

import org.apache.axis.AxisEngine;
import org.apache.axis.AxisFault;
import org.apache.axis.ConfigurationException;
import org.apache.axis.MessageContext;
import org.apache.axis.description.OperationDesc;
import org.apache.axis.description.ServiceDesc;
import org.apache.axis.handlers.soap.SOAPService;
import org.apache.axis.server.AxisServer;
import org.apache.axis.transport.http.AxisServlet;
import org.apache.axis.utils.XMLUtils;
import org.jboss.logging.Logger;
import org.jboss.mx.util.MBeanProxy;
import org.jboss.mx.util.MBeanProxyCreationException;
import org.jboss.mx.util.MBeanServerLocator;
import org.jboss.webservice.AxisServiceMBean;
import org.jboss.webservice.PortComponentInfo;
import org.jboss.webservice.ServiceDeployer;
import org.w3c.dom.Document;

import javax.management.MBeanServer;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Iterator;

/**
 * An JBossWS AxisServlet.
 *
 * It gets the server config via JMX from the AxisService.
 * Supports URL wsdl lookup with imports
 *
 * @author Thomas.Diesler@jboss.org
 * @since 15-April-2004
 */
public class ServiceEndpointServlet extends AxisServlet
{
   // provide logging
   protected final Logger log = Logger.getLogger(getClass());

   /** Denies access to the generic Axis way of accessing services
    *
    *    /context-root/services/ServiceName
    *
    * J2EE web services are deployed within their own context-root that may have authentication
    * requirements.
    */
   private boolean assertServiceAccess(HttpServletRequest req, HttpServletResponse res)
           throws IOException
   {
      String contextPath = req.getContextPath();
      String requestURI = req.getRequestURI();
      String pathInfo = req.getPathInfo();

      if (requestURI.startsWith(contextPath + "/services/") && "/Version".equals(pathInfo) == false)
      {
         reportTrouble(new IllegalAccessException(requestURI), res, res.getWriter());
         return false;
      }

      return true;
   }

   /**
    * Reject POST requests to '/ws4ee/services/someService'
    */
   public void doPost(HttpServletRequest req, HttpServletResponse res) throws ServletException, IOException
   {
      if (assertServiceAccess(req, res) == false)
         return;

      super.doPost(req, res);
   }

   /**
    * Process GET requests.
    */
   public void doGet(HttpServletRequest req, HttpServletResponse res)
           throws ServletException, IOException
   {
      if (assertServiceAccess(req, res) == false)
         return;

      String url = req.getRequestURL().toString();
      String queryString = req.getQueryString();
      log.debug("doGet: " + url + (queryString != null ? "?" + queryString : ""));

      PrintWriter writer = res.getWriter();
      try
      {
         AxisEngine engine = getEngine();

         // Get the SOAP servie
         String serviceName = getServiceName(req);
         SOAPService service = (serviceName != null ? engine.getService(serviceName) : null);

         boolean wsdlRequested = req.getParameter("wsdl") != null || req.getParameter("WSDL") != null;
         boolean listRequested = req.getParameter("list") != null || req.getParameter("LIST") != null;

         if (!listRequested && !wsdlRequested)
         {
            log.debug("Report available services");

            // If the user requested the servlet (i.e. /ws4ee/services)
            // with no service name, present the user with a list of deployed services to be helpful
            // Don't do this if we are doing WSDL or list.
            reportAvailableServices(res, writer, req);
            return;
         }

         if (service == null)
         {
            log.error("Cannot get axis service: " + serviceName);
            reportCantGetAxisService(req, res, writer);
            return;
         }

         if (listRequested)
         {
            log.debug("Process the list request");
            processListRequest(res, writer);
            return;
         }

         // get message context w/ various properties set
         MessageContext msgContext = createMessageContext(engine, req, res);

         // we found the service, so we can set it in the msg context
         // whoever comes after, won't have to retry finding it
         if (service != null)
            msgContext.setTargetService(serviceName);

         String transportURL = getTransportURL(req, serviceName);
         if (transportURL != null)
         {
            msgContext.setProperty(MessageContext.TRANS_URL, transportURL);
            log.debug("Set transport.url=" + transportURL);
         }

         if (wsdlRequested)
         {
            String wsdlResource = req.getParameter("resource");
            if (wsdlResource != null)
            {
               log.debug("Process wsdl import request: " + wsdlResource);
               msgContext.setProperty(MessageContext.WSDLGEN_RESOURCE, wsdlResource);
            }
            else
            {
               log.debug("Process wsdl request");
            }

            processWsdlRequest(msgContext, res, writer);
            return;
         }

         // If nothing else was done previously
         log.debug("Report service info");
         reportServiceInfo(res, writer, service, serviceName);
      }
      catch (AxisFault fault)
      {
         reportTrouble(fault, res, writer);
      }
      catch (Exception e)
      {
         reportTrouble(e, res, writer);
      }
      finally
      {
         // Make sure the MessageContext is removed from the calling ThreadLocal
         AxisEngine.setCurrentMessageContext(null);
         writer.close();
      }
   }

   /**
    * Get the service name as it is known to Axis
    * <p/>
    * For JSE service endpoints it is obtained from the generated init parameter in web.xml
    */
   protected String getServiceName(HttpServletRequest req)
   {
      String serviceName = null;
      if (req.getRequestURI().equals(req.getContextPath() + "/services/Version"))
         serviceName = "Version";
      else
         serviceName = getServletConfig().getInitParameter(ServiceDeployer.INIT_PARAM_WEBSERVICE_ID);

      return serviceName;
   }

   /**
    * Get the transport URL
    * <p/>
    * For JSE service endpoints
    * e.g. http://localhost:8080/whatever-url-mapping
    */
   protected String getTransportURL(HttpServletRequest req, String serviceName)
   {
      return req.getRequestURI();
   }

   protected void setupHTMLResponseHeader(HttpServletResponse res, PrintWriter writer)
   {
      writer.println("<head>");
      writer.println("<meta http-equiv='Content-Type content='text/html; charset=iso-8859-1'>");
      writer.println("<title>JBossWS</title>");
      writer.println("<link rel='stylesheet' href='/ws4ee/styles.css'>");
      writer.println("</head>");
   }

   /**
    * This method lists the available services; it is called when there is
    * nothing to execute on a GET
    */
   protected void reportAvailableServices(HttpServletResponse res, PrintWriter writer, HttpServletRequest req)
           throws ConfigurationException, AxisFault
   {
      AxisEngine engine = getEngine();

      res.setContentType("text/html");
      setupHTMLResponseHeader(res, writer);
      writer.println("<h2>And now... Some Services</h2>");
      writer.println("<ul>");

      // Get the JBossWS AxisService
      AxisServiceMBean axisService = null;
      try
      {
         MBeanServer server = MBeanServerLocator.locateJBoss();
         axisService = (AxisServiceMBean)MBeanProxy.get(AxisServiceMBean.class, AxisServiceMBean.OBJECT_NAME, server);
      }
      catch (MBeanProxyCreationException e)
      {
         throw new IllegalStateException("Cannot obtain: " + AxisServiceMBean.OBJECT_NAME);
      }

      Iterator it = engine.getConfig().getDeployedServices();
      while (it.hasNext())
      {
         ServiceDesc sd = (ServiceDesc)it.next();
         String serviceName = sd.getName();

         PortComponentInfo pcInfo = axisService.getPortComponentInfo(serviceName);
         String serviceURL = null;
         if (pcInfo != null)
         {
            serviceURL = pcInfo.getServiceEndpointURL();
         }
         else
         {
            serviceURL = getWebappBase(req) + "/services/" + serviceName;
         }

         StringBuffer sb = new StringBuffer("<li>" + serviceName + "<a href='" + serviceURL + "?wsdl'>&nbsp;<i>(wsdl)</i></a></li>");

         ArrayList operations = sd.getOperations();
         if (!operations.isEmpty())
         {
            sb.append("<ul>");
            for (Iterator itOp = operations.iterator(); itOp.hasNext();)
            {
               OperationDesc desc = (OperationDesc)itOp.next();
               sb.append("<li>" + desc.getName() + "</li>");
            }
            sb.append("</ul>");
         }
         writer.println(sb.toString());
      }
      writer.println("</ul>");
   }

   /**
    * Handle a wsdl request
    */
   protected void processWsdlRequest(MessageContext msgContext, HttpServletResponse res, PrintWriter writer)
           throws AxisFault
   {
      String serviceURL = (String)msgContext.getProperty(MessageContext.TRANS_URL);

      AxisEngine engine = getEngine();
      engine.generateWSDL(msgContext);

      Document doc = (Document)msgContext.getProperty("WSDL");
      if (doc == null)
         throw new AxisFault("Cannot get wsdl document for service: " + serviceURL);

      res.setContentType("text/xml");
      XMLUtils.DocumentToWriter(doc, writer);
   }

   /**
    * Get an AxisServer with the EngineConfiguration from the JMX AxisService
    */
   public AxisServer getEngine() throws AxisFault
   {
      try
      {
         MBeanServer server = MBeanServerLocator.locateJBoss();
         axisServer = (AxisServer)server.getAttribute(AxisServiceMBean.OBJECT_NAME, "AxisServer");
      }
      catch (Exception e)
      {
         log.warn("Cannot access AxisService, using default server config");
         axisServer = super.getEngine();
      }

      return axisServer;
   }

}

==> jboss-4.0.2-src.tar.bz2: jboss-4.0.2-src/webservice/src/main/org/jboss/webservice/server/ServiceEndpointServlet.java <==
/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.webservice.server;

// $Id: ServiceEndpointServlet.java,v 1.8.2.5 2005/04/12 16:07:49 starksm Exp $

import org.jboss.axis.AxisEngine;
import org.jboss.axis.AxisFault;
import org.jboss.axis.MessageContext;
import org.jboss.axis.handlers.soap.SOAPService;
import org.jboss.logging.Logger;
import org.jboss.webservice.ServiceDeployer;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.PrintWriter;

/**
 * An JBossWS AxisServlet.
 *
 * It gets the server config via JMX from the AxisService.
 * Supports URL wsdl lookup with imports
 *
 * @author Thomas.Diesler@jboss.org
 * @since 15-April-2004
 */
public class ServiceEndpointServlet extends AbstractServlet
{
   /** @since 4.0.2 */
   static final long serialVersionUID = 3806847653370058302L;

   // provide logging
   protected final Logger log = Logger.getLogger(getClass());

   /**
    * Process GET requests.
    */
   public void doGet(HttpServletRequest req, HttpServletResponse res)
           throws ServletException, IOException
   {
      String url = req.getRequestURL().toString();
      String queryString = req.getQueryString();
      log.debug("doGet: " + url + (queryString != null ? "?" + queryString : ""));

      PrintWriter writer = res.getWriter();
      try
      {
         AxisEngine engine = getEngine();

         // Get the SOAP servie
         String serviceName = getServiceName(req);
         SOAPService service = (serviceName != null ? engine.getService(serviceName) : null);

         boolean wsdlRequested = req.getParameter("wsdl") != null || req.getParameter("WSDL") != null;

         if (!wsdlRequested)
         {
            log.debug("Report available services");

            // If the user requested the servlet (i.e. /ws4ee/services)
            // with no service name, present the user with a list of deployed services to be helpful
            // Don't do this if we are doing WSDL or list.
            reportAvailableServices(res, writer, req);
            return;
         }

         if (service == null)
         {
            log.error("Cannot get axis service: " + serviceName);
            reportCantGetAxisService(req, res, writer);
            return;
         }

         // get message context w/ various properties set
         MessageContext msgContext = createMessageContext(engine, req, res);

         // we found the service, so we can set it in the msg context
         // whoever comes after, won't have to retry finding it
         if (service != null)
            msgContext.setTargetService(serviceName);

         String transportURL = getTransportURL(req, serviceName);
         if (transportURL != null)
         {
            msgContext.setProperty(MessageContext.TRANS_URL, transportURL);
            log.debug("Set transport.url=" + transportURL);
         }

         if (wsdlRequested)
         {
            String wsdlResource = req.getParameter("resource");
            if (wsdlResource != null)
            {
               log.debug("Process wsdl import request: " + wsdlResource);
               msgContext.setProperty(MessageContext.WSDLGEN_RESOURCE, wsdlResource);
            }
            else
            {
               log.debug("Process wsdl request");
            }

            processWsdlRequest(msgContext, res, writer);
            return;
         }

         // If nothing else was done previously
         log.debug("Report service info");
         reportServiceInfo(res, writer, service, serviceName);
      }
      catch (AxisFault fault)
      {
         reportTrouble(fault, res, writer);
      }
      catch (Exception e)
      {
         reportTrouble(e, res, writer);
      }
      finally
      {
         // Make sure the MessageContext is removed from the calling ThreadLocal
         AxisEngine.setCurrentMessageContext(null);
         writer.close();
      }
   }

   /**
    * Get the service name as it is known to Axis
    * <p/>
    * For JSE service endpoints it is obtained from the generated init parameter in web.xml
    */
   protected String getServiceName(HttpServletRequest req)
   {
      String serviceName = getServletConfig().getInitParameter(ServiceDeployer.INIT_PARAM_WEBSERVICE_ID);
      return serviceName;
   }
}
